import { Link, useParams, useHistory } from 'react-router-dom'

import Page from 'components/page'
import { useTranslation } from 'react-i18next'
import {
  Button,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Paper,
} from '@mui/material'
import { Guest, Role } from 'interfaces'
import SponsorInfoButtons from 'routes/components/sponsorInfoButtons'
import { format } from 'date-fns'
import React from 'react'
import { submitJsonOpts } from '../../../../utils'

type GuestInfoParams = {
  pid: string
}

interface RoleLineProps {
  role: Role
  pid: string
}

type GuestInfoProps = {
  guest: Guest
  roles: Role[]
}

const RoleLine = ({ role, pid }: RoleLineProps) => {
  const [t, i18n] = useTranslation('common')
  return (
    <TableRow sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
      <TableCell align="left">
        {i18n.language === 'en' ? role.name_en : role.name_nb}
      </TableCell>
      <TableCell component="th" scope="row">
        {role.start_date ? format(role.start_date, 'yyyy-MM-dd') : null} -{' '}
        {format(role.end_date, 'yyyy-MM-dd')}
      </TableCell>
      <TableCell align="left">
        {i18n.language === 'en' ? role.ou_en : role.ou_nb}
      </TableCell>
      <TableCell>
        <Button
          variant="contained"
          component={Link}
          to={`/sponsor/guest/${pid}/roles/${role.id}`}
        >
          {t('sponsor.choose')}
        </Button>
      </TableCell>
    </TableRow>
  )
}

export default function GuestInfo({ guest, roles }: GuestInfoProps) {
  const { pid } = useParams<GuestInfoParams>()
  const [t] = useTranslation(['common'])
  const history = useHistory()

  // Resending the invitation does not cause a change in the state, so nothing needs to be updated after the call
  const resend = () => {
    fetch(`/api/ui/v1/invite/${guest.pid}/resend`, submitJsonOpts('PATCH', {}))
      .then((res) => {
        if (!res.ok) {
          return null
        }
        return res.text()
      })
      .then((result) => {
        if (result !== null) {
          console.log('result', result)
        }
      })
      .catch((error) => {
        console.log('error', error)
      })
  }

  const handleCancel = () => {
    // There is no body for this request, but using submitJsonOpts still to
    // set the CSRF-token
    fetch(
      `/api/ui/v1/invite/?person_id=${guest.pid}`,
      submitJsonOpts('DELETE', {})
    )
      .then((res) => {
        if (!res.ok) {
          return null
        }
        return res.text()
      })
      .then((result) => {
        if (result !== null) {
          // The invite for the guest has been cancelled, send the user back to the sponsor front page
          history.push('/sponsor')
        }
      })
      .catch((error) => {
        // TODO User should get some feedback telling him something failed
        console.log('error', error)
      })
  }

  return (
    <Page>
      <SponsorInfoButtons to="/sponsor" name={`${guest.first} ${guest.last}`} />
      <h4>{t('guestInfo.contactInfo')}</h4>
      <TableContainer component={Paper}>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead sx={{ backgroundColor: 'primary.light' }}>
            <TableRow>
              <TableCell align="left">{t('guestInfo.contactInfo')}</TableCell>
              <TableCell />
            </TableRow>
          </TableHead>
          <TableBody>
            <TableRow>
              <TableCell align="left">{t('input.fullName')}</TableCell>
              <TableCell align="left">
                {`${guest.first} ${guest.last}`}
              </TableCell>
            </TableRow>
            <TableRow>
              <TableCell align="left">{t('input.email')}</TableCell>
              <TableCell align="left">{guest.email}</TableCell>
            </TableRow>
            <TableRow>
              <TableCell align="left">{t('input.nationalIdNumber')}</TableCell>
              <TableCell align="left">{guest.fnr}</TableCell>
            </TableRow>
            <TableRow>
              <TableCell align="left">{t('input.mobilePhone')}</TableCell>
              <TableCell align="left">{guest.mobile}</TableCell>
            </TableRow>
          </TableBody>
        </Table>
      </TableContainer>

      {/* If the guest has not completed the registration process, he should have an invitation he has not responded to */}
      {!guest.registered && (
        <>
          <Button sx={{ marginTop: '0.5rem' }} onClick={resend}>
            {t('button.resendInvitation')}
          </Button>
          <Button onClick={handleCancel}>{t('button.cancelInvitation')}</Button>
        </>
      )}

      <h4>{t('guestInfo.roleInfoHead')}</h4>
      <h5>
        {t('guestInfo.roleInfoBody')}
        <Button
          variant="contained"
          component={Link}
          to={`/sponsor/guest/${pid}/newrole`}
        >
          {t('sponsor.addRole')}
        </Button>
      </h5>

      <TableContainer component={Paper}>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead sx={{ backgroundColor: 'primary.light' }}>
            <TableRow>
              <TableCell align="left">{t('common:role')}</TableCell>
              <TableCell align="left">{t('common:period')}</TableCell>
              <TableCell align="left">{t('common:ou')}</TableCell>
              <TableCell align="left">{t('common:choice')}</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {roles.map((role) => (
              <RoleLine key={role.id} pid={pid} role={role} />
            ))}
          </TableBody>
        </Table>
      </TableContainer>
    </Page>
  )
}
